package provide BTS_TCLSHELL 0.1

# Required Packages
package require BTS_BASE;
package require BTS_GUI;

# ***********************************************************************
#	Init function: Sets up variables
# ***********************************************************************

namespace eval ::bts::shell {	
	namespace import ::bts::bts;
	#enable the tcl shell in the BTS GUI
	set ::bts::gui::bts_gui(tcl_shell_enable) 1;
	
	set shell(prompt) "BTS> "
}

proc ::bts::shell::build_tclshell {notebook tab tab_frame} {
	variable shell;
	
	
	set shell(text) [text $tab_frame.log_window -setgrid false -fg black -font {Courier}  \
					   -yscrollcommand "$tab_frame.log_scroll set"];
	scrollbar $tab_frame.log_scroll -orient vertical -command "$shell(text) yview";
	place $tab_frame.log_scroll -relheight 1 -width 15 -rely 0 -relx 1 -x -15 ;
	place $shell(text) -relheight 1 -relwidth 1 -width -15 -rely 0 -relx 0;
	
	# Text tags give script output, command errors, command
	# results, and the prompt a different appearance
	
	$shell(text) tag configure prompt -underline true;
	$shell(text) tag configure result -foreground purple;
	$shell(text) tag configure error -foreground red;
	$shell(text) tag configure output -foreground blue;
	
	# Insert the prompt and initialize the limit mark
	
	
	$shell(text) insert insert $shell(prompt) prompt;
	$shell(text) mark set limit insert;
	$shell(text) mark gravity limit left;
	focus $shell(text);
	
	
	# Key bindings that limit input and eval things. The break in
	# the bindings skips the default Text binding for the event.
	
	bind $shell(text) <Return> {::bts::shell::EvalTypein ; break}
	bind $shell(text) <BackSpace> {
		if {[%W tag nextrange sel 1.0 end] != ""} {
			%W delete sel.first sel.last;
		} elseif {[%W compare insert > limit]} {
			%W delete insert-1c;
			%W see insert;
		}
		break
	}
	bind $shell(text) <Key> {
		if [%W compare insert < limit] {
			%W mark set insert end;
		}
	}
	
	$notebook itemconfigure $tab -raisecmd "::bts::bts gui raise_tab lower bts_msgs";
	
	set shell(slave) [SlaveInit shell];
	
}

# Evaluate everything between limit and end as a Tcl command

proc ::bts::shell::EvalTypein {} {
	variable shell
	$shell(text) insert insert \n
	set command [$shell(text) get limit end]
	if [info complete $command] {
		$shell(text) mark set limit insert
		Eval $command
	}
}

# Echo the command and evaluate it
proc ::bts::shell::EvalEcho {command} {
	variable eval
	$shell(text) mark set insert end;
	$shell(text) insert insert $command\n;
	Eval $command;
}

# Evaluate a command and display its result
proc ::bts::shell::Eval {command} {
	variable shell
	$shell(text) mark set insert end
	if [catch {$shell(slave) eval $command} result] {
		$shell(text) insert insert $result error
	} else {
		$shell(text) insert insert $result result
	}
	if {[$shell(text) compare insert != "insert linestart"]} {
		$shell(text) insert insert \n
	}
	$shell(text) insert insert $shell(prompt) prompt
	$shell(text) see insert
	$shell(text) mark set limit insert
	return
}

# Create and initialize the slave interpreter
proc ::bts::shell::SlaveInit {slave} {
	interp create $slave
#		load {} Tk $slave
	interp alias $slave bts {} ::bts::bts;
	interp alias $slave reset {} ::bts::shell::ResetAlias $slave;
	interp alias $slave puts {} ::bts::shell::PutsAlias $slave;

	return $slave;
}

# The reset alias deletes the slave and starts a new one
proc ::bts::shell::ResetAlias {slave} {
	interp delete $slave
	SlaveInit $slave
}

# The puts alias puts stdout and stderr into the text widget
	
proc ::bts::shell::PutsAlias {slave args} {
	variable shell;
	if {[llength $args] > 3} {
		error "invalid arguments"
	}
	set newline "\n"
	if {[string match "-nonewline" [lindex $args 0]]} {
		set newline ""
		set args [lreplace $args 0 0]
	}
	if {[llength $args] == 1} {
		set chan stdout
		set string [lindex $args 0]$newline
	} else {
		set chan [lindex $args 0]
		set string [lindex $args 1]$newline
	}
	if [regexp (stdout|stderr) $chan] {
		global eval
		$shell(text) mark gravity limit right
		$shell(text) insert limit $string output
		$shell(text) see limit
		$shell(text) mark gravity limit left
	} else {
		puts -nonewline $chan $string
	}
}
